/**
 * Definition for singly-linked list.
 * struct ListNode {
 *     int val;
 *     ListNode *next;
 *     ListNode(int x) : val(x), next(NULL) {}
 * };
 */
///////////////////////////////////////////////////////////////////////////////////
// Category: Removal
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Delete Node in a Linked List [237]
 *
 * Write a function to delete a node (except the tail) in a singly linked list,
 * given only access to that node.
 * Supposed the linked list is 1 -> 2 -> 3 -> 4 and you are given the third node
 * with value 3, the linked list should become 1 -> 2 -> 4 after calling your function.
 */
void deleteNode(ListNode *node)
{
	ListNode *p = node->next;
	node->val = p->val;
	node->next = p->next;
	delete p;
}

// --------------------------------------------------------------------------------
/**
 * 2. Remove Nth Node From End of List [19]
 *
 * Given a linked list, remove the n-th node from the end of list and return its head.
 * @example
 *     Given linked list: 1->2->3->4->5, and n = 2.
 *     After removing the second node from the end, the linked list becomes 1->2->3->5.
 * @endexample
 * Note:
 * Given n will always be valid. Try to do this in one pass.
 */

/**
 * The first pointer advances the list by n+1 steps from the beginning, while the
 * second pointer starts from the beginning of the list. Now, both pointers are exactly
 * separated by n nodes apart. We maintain this constant gap by advancing both pointers
 * together until the first pointer arrives past the last node. The second pointer will
 * be pointing at the nth node counting from the last. We relink the next pointer of
 * the node referenced by the second pointer to point to the node's next next node.
 */
ListNode* removeNthFromEnd(ListNode *head, int n)
{
	if (n <= 0)
		return head;
	ListNode preHead(0);
	preHead.next = head;
	ListNode *p = &preHead, *q = p;
	int i = 0;
	for (; p != NULL && i <= n; ++i)
		p = p->next;
	if (i <= n) // n is greater than the list length
		return head;
	while (p != NULL)
	{
		p = p->next;
		q = q->next;
	}
	p = q->next;
	q->next = p->next;
	delete p;
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 3. Remove Linked List Elements [203]
 *
 * Remove all elements from a linked list of integers that have value val.
 * @example
 *     Given: 1 -> 2 -> 6 -> 3 -> 4 -> 5 -> 6, val = 6
 *     Return: 1 -> 2 -> 3 -> 4 -> 5
 * @endexample
 */
ListNode* removeElements(ListNode *head, int val)
{
	ListNode preHead(0), *p = &preHead;
	preHead.next = head;
	while (p != NULL)
	{
		while (p->next != NULL && p->next->val == val)
		{
			ListNode *q = p->next;
			p->next = q->next;
			delete q;
		}
		p = p->next;
	}
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 4. Remove Duplicates from Sorted List [83]
 *
 * Given a sorted linked list, delete all duplicates such that each element appear only once.
 * @example
 *     Given 1->1->2, return 1->2.
 * @endexample
 * @example
 *     Given 1->1->2->3->3, return 1->2->3.
 * @endexample
 */
ListNode* deleteDuplicates(ListNode *head)
{
	ListNode *p = head, *q = p != NULL ? p->next : NULL;
	while (q != NULL)
	{
		while (q != NULL && q->val == p->val)
		{
			q = q->next;
			delete p->next;
			p->next = q;
		}
		p = q;
		if (p != NULL)
			q = p->next;
	}
	return head;
}

// --------------------------------------------------------------------------------
/**
 * 5. Remove Duplicates from Sorted List II [82]
 *
 * Given a sorted linked list, delete all nodes that have duplicate numbers,
 * leaving only distinct numbers from the original list.
 * @example
 *     Given 1->2->3->3->4->4->5, return 1->2->5.
 * @endexample
 * @example
 *     Given 1->1->1->2->3, return 2->3.
 * @endexample
 */
ListNode* deleteDuplicates(ListNode *head)
{
	ListNode preHead(0);
	preHead.next = head;
	ListNode *r = &preHead, *p = head, *q = p != NULL ? p->next : NULL;
	while (q != NULL)
	{
		bool isDuplicate = false;
		while (q != NULL && q->val == p->val)
		{
			q = q->next;
			delete p->next;
			p->next = q;
			isDuplicate = true;
		}
		if (isDuplicate)
		{
			delete p;
			r->next = q;
		}
		else
			r = p;
		p = q;
		if (p != NULL)
			q = p->next;
	}
	return preHead.next;
}





///////////////////////////////////////////////////////////////////////////////////
// Category: Reorder
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Reverse Linked List [206]
 *
 * Reverse a singly linked list. A linked list can be reversed either
 * iteratively or recursively. Could you implement both?
 */
ListNode* reverseList(ListNode *head)
{
	ListNode *pPrev = NULL, *pCur = head;
	while (pCur != NULL)
	{
		ListNode *pNext = pCur->next;
		pCur->next = pPrev;
		pPrev = pCur;
		pCur = pNext;
	}
	return pPrev;
}

ListNode* reverseList(ListNode *head)
{
	if (head == NULL || head->next == NULL)
		return head;
	ListNode *last = reverseList(head->next);
	head->next->next = head;
	head->next = NULL;
	return last;
}

// --------------------------------------------------------------------------------
/**
 * 2. Reverse Linked List II [92]
 *
 * Reverse a linked list from position m to n. Do it in-place and in one-pass.
 * @example
 *     Given 1->2->3->4->5->NULL, m = 2 and n = 4,
 *     return 1->4->3->2->5->NULL.
 * @endexample
 * Note:
 * Given m, n satisfy the following condition:
 * 1 <= m <= n <= length of list.
 */
ListNode* reverseBetween(ListNode *head, int m, int n)
{
	if (head == NULL || m == n)
		return head;
	n -= m;
	ListNode preHead(0);
	preHead.next = head;
	ListNode *p = &preHead, *q = head, *r = q->next;
	while (--m > 0)
		p = p->next;
	q = p->next;
	while (n-- > 0)
	{
		r = q->next;
		q->next = r->next;
		r->next = p->next;
		p->next = r;
	}
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 3. Swap Nodes in Pairs [24]
 *
 * Given a linked list, swap every two adjacent nodes and return its head.
 * @example
 *     Given 1->2->3->4, you should return the list as 2->1->4->3.
 * @endexample
 * Your algorithm should use only constant space. You may not modify the
 * values in the list, only nodes itself can be changed.
 */
ListNode* swapPairs(ListNode *head)
{
	ListNode preHead(0);
	preHead.next = head;
	ListNode *p = &preHead, *q = head;
	while (q != NULL && q->next != NULL)
	{
		p->next = q->next;
		q->next = q->next->next;
		p->next->next = q;
		p = q;
		q = q->next;
	}
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 4. Reverse Nodes in k-Group [25]
 *
 * Given a linked list, reverse the nodes of a linked list k
 * at a time and return its modified list. k is a positive integer
 * and is less than or equal to the length of the linked list.
 * If the number of nodes is not a multiple of k then left-out nodes
 * in the end should remain as it is.
 * You may not alter the values in the nodes, only nodes itself may be changed.
 * Only constant memory is allowed.
 * For example,
 * Given this linked list: 1 -> 2 -> 3 -> 4 -> 5
 * For k = 2, you should return: 2 -> 1 -> 4 -> 3 -> 5
 * For k = 3, you should return: 3 -> 2 -> 1 -> 4 -> 5
 */
ListNode* reverseKGroup(ListNode *head, int k)
{
	if (k == 1)
		return head;
	ListNode preHead(0);
	preHead.next = head;
	ListNode *p = head, *q = NULL, *r = &preHead;
	while (true)
	{
		int cnt = 0;
		for (p = r->next; p != NULL && cnt < k; p = p->next)
			++cnt;
		if (cnt < k)
			break;
		p = r->next;
		q = p->next;
		for (int j = 1; j < k; ++j)
		{
			p->next = q->next;
			q->next = r->next;
			r->next = q;
			q = p->next;
		}
		r = p;
	}
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 5. Rotate List [61]
 *
 * Given a list, rotate the list to the right by k places, where k is non-negative.
 * @example
 *     Given 1->2->3->4->5->NULL and k = 2,
 *     return 4->5->1->2->3->NULL.
 * @endexample
 */
ListNode* rotateRight(ListNode *head, int k)
{
	if (head == NULL || k <= 0)
		return head;
	int L = 1;
	ListNode *t = head;
	for (; t->next != NULL; t = t->next)
		++L;
	k %= L;
	if (k == 0)
		return head;
	ListNode *p = head, *q = p;
	for (int i = 0; i < L - k; ++i)
	{
		q = p;
		p = p->next;
	}
	t->next = head;
	q->next = NULL;
	return p;
}

// --------------------------------------------------------------------------------
/**
 * 6. Partition List [86]
 *
 * Given a linked list and a value x, partition it such that all nodes
 * less than x come before nodes greater than or equal to x.
 * You should preserve the original relative order of the nodes in each of the two partitions.
 * @example
 *     Given 1->4->3->2->5->2 and x = 3,
 *     return 1->2->2->4->3->5.
 * @endexample
 */
ListNode* partition(ListNode *head, int x)
{
	ListNode lo(0), hi(0);
	ListNode *p = &lo, *q = &hi;
	while (head != NULL)
	{
		if (head->val < x)
			p = p->next = head;
		else
			q = q->next = head;
		head = head->next;
	}
	p->next = hi.next;
	q->next = NULL;
	return lo.next;
}

// --------------------------------------------------------------------------------
/**
 * 7. Odd Even Linked List [328]
 *
 * Given a singly linked list, group all odd nodes together followed by the even nodes.
 * Please note here we are talking about the node number and not the value in the nodes.
 * You should try to do it in place.
 * The program should run in O(1) space complexity and O(nodes) time complexity.
 * @example
 *     Given 1->2->3->4->5->NULL,
 *     return 1->3->5->2->4->NULL.
 * @endexample
 * Note:
 * The relative order inside both the even and odd groups should remain as it was in the input.
 * The first node is considered odd, the second node even and so on ...
 */
ListNode* oddEvenList(ListNode *head)
{
	if (head == NULL)
		return NULL;
	ListNode *odd = head, *evenhead = head->next, *even = evenhead;
	while (even != NULL && even->next != NULL)
	{
		odd->next = odd->next->next;
		even->next = even->next->next;
		odd = odd->next;
		even = even->next;
	}
	odd->next = evenhead;
	return head;
}

// --------------------------------------------------------------------------------
/**
 * 8. Insertion Sort List [147]
 *
 * Sort a linked list using insertion sort.
 */
ListNode* insertionSortList(ListNode *head)
{
	if (head == NULL || head->next == NULL)
		return head;
	ListNode preHead(0);
	preHead.next = head;
	ListNode *p = NULL, *q = head; // q is the tail node of ordered sequence
	while (q->next != NULL)
	{
		p = &preHead;
		while (p->next != q && p->next->val < q->next->val)
			p = p->next;
		if (p->next == q && q->val <= q->next->val) // insert to tail means no need to insert
			q = q->next;
		// 0 -> 3 -> 6 -> 8 -> 1 -> 2    or    0 -> 3 -> 6 -> 8 -> 7 -> 2
		// p              q    r         or              p    q    r
		else // insert q->next between p and p->next
		{
			ListNode *r = q->next;
			q->next = r->next;
			r->next = p->next;
			p->next = r;
		}
	}
	return preHead.next;
}






///////////////////////////////////////////////////////////////////////////////////
// Category: fast slow pointers
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Sort List [148]
 *
 * Sort a linked list in O(n log n) time using constant space complexity.
 */
ListNode* sortList(ListNode *head)
{
	if (head == NULL || head->next == NULL)
		return head;
	ListNode *slow = head, *fast = head->next->next;
	while (fast != NULL && fast->next != NULL)
	{
		fast = fast->next->next;
		slow = slow->next;
	}
	fast = slow->next;
	slow->next = NULL;
	slow = sortList(head);
	fast = sortList(fast);
	return mergeTwoLists(slow, fast); // see [21]
}

// --------------------------------------------------------------------------------
/**
 * 2. Linked List Cycle [141]
 *
 * Given a linked list, determine if it has a cycle in it.
 * Follow up:
 * Can you solve it without using extra space?
 */
bool hasCycle(ListNode *head)
{
	ListNode *p = head, *q = p;
	while (p != NULL && p->next != NULL)
	{
		p = p->next->next;
		q = q->next;
		if (p == q)
			return true;
	}
	return false;
}

// --------------------------------------------------------------------------------
/**
 * 3. Linked List Cycle II [142]
 *
 * Given a linked list, return the node where the cycle begins.
 * If there is no cycle, return null.
 * Note:
 * Do not modify the linked list.
 * Follow up:
 * Can you solve it without using extra space?
 */

/**
 * Using two pointers, one of them one step at a time. another pointer each take two steps.
 * Suppose the first meet at step k,the length of the Cycle is r. => 2k - k = nr, k = nr
 * Now, the distance between the start node of list and the start node of cycle is s.
 * the distance between the start of list and the first meeting node is k (the pointer
 * which wake one step at a time waked k steps).the distance between the start node of
 * cycle and the first meeting node is m. => s = k - m, s = nr - m = (n-1)r + (r-m),
 * here we takes n = 1, hence s = r - m, using one pointer start from the start node of list,
 * another pointer start from the first meeting node, all of them wake one step at a time,
 * the first time they meeting each other is the start of the cycle.
 */
ListNode* detectCycle(ListNode *head)
{
	ListNode *p = head, *q = p;
	while (p != NULL && p->next != NULL)
	{
		p = p->next->next;
		q = q->next;
		if (p == q)
		{
			p = head;
			while (true)
			{
				if (p == q)
					return p;
				p = p->next;
				q = q->next;
			}
		}
	}
	return NULL;
}

// --------------------------------------------------------------------------------
/**
 * 4. Reorder List [143]
 *
 * Given a singly linked list L: L0 -> L1 -> ... -> Ln-1 -> Ln,
 * reorder it to: L0 -> Ln -> L1 -> Ln-1 -> L2 -> Ln-2 -> ...
 * You must do this in-place without altering the nodes' values.
 * @example
 *     Given {1,2,3,4}, reorder it to {1,4,2,3}.
 * @endexample
 */
void reorderList(ListNode *head)
{
	if (head == NULL)
		return;
	ListNode *p = head, *q = p;
	while (p != NULL && p->next != NULL)
	{
		p = p->next->next;
		q = q->next;
	}
	p = head;
	q->next = reverseList(q->next);
	ListNode *r = q;
	q = q->next;
	r->next = NULL;
	while (q != NULL)
	{
		r = q;
		q = q->next;
		r->next = p->next;
		p->next = r;
		p = r->next;
	}
}





///////////////////////////////////////////////////////////////////////////////////
// Category: Two Lists
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Add Two Numbers [2]
 *
 * You are given two non-empty linked lists representing two non-negative integers.
 * The digits are stored in reverse order and each of their nodes contain a single digit.
 * Add the two numbers and return it as a linked list.
 * You may assume the two numbers do not contain any leading zero,
 * except the number 0 itself.
 * @example
 *     Input: (2 -> 4 -> 3) + (5 -> 6 -> 4)
 *     Output: 7 -> 0 -> 8
 *     Explanation: 342 + 465 = 807.
 * @endexample
 */
ListNode *addTwoNumbers(ListNode *l1, ListNode *l2)
{
	ListNode preHead(0), *p = &preHead;
	int extra = 0;
	while (l1 != NULL || l2 != NULL || extra > 0)
	{
		int sum = (l1 != NULL ? l1->val : 0) + (l2 != NULL ? l2->val : 0) + extra;
		extra = sum / 10;
		p->next = new ListNode(sum % 10);
		p = p->next;
		l1 = l1 != NULL ? l1->next : l1;
		l2 = l2 != NULL ? l2->next : l2;
	}
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 2. Add Two Numbers II [445]
 *
 * You are given two non-empty linked lists representing two non-negative integers.
 * The most significant digit comes first and each of their nodes contain a single
 * digit. Add the two numbers and return it as a linked list. You may assume the
 * two numbers do not contain any leading zero, except the number 0 itself.
 * @example
 *     Input: (7 -> 2 -> 4 -> 3) + (5 -> 6 -> 4)
 *     Output: 7 -> 8 -> 0 -> 7
 * @endexample
 * Follow up:
 * What if you cannot modify the input lists? In other words, reversing the lists
 * is not allowed.
 */
class Solution
{
public:
	ListNode* addTwoNumbers(ListNode *l1, ListNode *l2)
	{
		if (l1 == NULL || l2 == NULL)
			return l1 != NULL ? l1 : l2;
		int diff = 0, extra = 0;
		ListNode *p = l1, *q = l2, *head = NULL;
		while (p->next != NULL && q->next != NULL)
		{
			p = p->next;
			q = q->next;
		}
		bool flag = q->next == NULL;
		if (p->next != NULL)
			for (; p->next != NULL; p = p->next)
				++diff;
		else if (q->next != NULL)
			for (; q->next != NULL; q = q->next)
				++diff;
		extra = addTwoNodeVal(flag ? l1 : l2, flag ? l2 : l1, diff);
		if (extra == 1)
		{
			head = new ListNode(1);
			head->next = flag ? l1 : l2;
		}
		else
			head = flag ? l1 : l2;
		return head;
	}

private:
	int addTwoNodeVal(ListNode *l1, ListNode *l2, int d)
	{
		if (l1 == NULL)
			return 0;
		int sum = l1->val + (d > 0 ? 0 : l2->val) + addTwoNodeVal(l1->next, d > 0 ? l2 : l2->next, d-1);
		l1->val = sum % 10;
		return sum / 10;
	}
};

// --------------------------------------------------------------------------------
/**
 * 3. Merge Two Sorted Lists [21]
 *
 * Merge two sorted linked lists and return it as a new list.
 * The new list should be made by splicing together the nodes of the first two lists.
 * @example
 *     Input: 1->2->4, 1->3->4
 *     Output: 1->1->2->3->4->4
 * @endexample
 */
ListNode* mergeTwoLists(ListNode *l1, ListNode *l2)
{
	ListNode preHead(0);
	ListNode *tail = &preHead;
	
	while (l1 != NULL && l2 != NULL)
	{
		if (l1->val < l2->val)
		{
			tail->next = l1;
			l1 = l1->next;
		}
		else
		{
			tail->next = l2;
			l2 = l2->next;
		}
		tail = tail->next;
	}

	tail->next = l1 != NULL ? l1 : l2;
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 4. Merge k Sorted Lists [23]
 *
 * Merge k sorted linked lists and return it as one sorted list.
 * Analyze and describe its complexity.
 */
ListNode* mergeKLists(std::vector<ListNode*>& lists)
{
	int i = 0, n = static_cast<int>(lists.size());
	if (n == 0)
		return NULL;
	if (n == 1)
		return lists[0];
	ListNode preHead(0), *tail = &preHead;
	std::priority_queue<std::pair<int, int>, std::vector<std::pair<int, int> >, std::greater<std::pair<int, int> > > priorityQ;
	for (i = 0; i < n; ++i)
		if (lists[i] != NULL)
			priorityQ.push(std::pair<int, int>(lists[i]->val, i));
	while (!priorityQ.empty())
	{
		i = priorityQ.top().second;
		priorityQ.pop();
		tail->next = lists[i];
		tail = tail->next;
		if (lists[i]->next != NULL)
		{
			lists[i] = lists[i]->next;
			priorityQ.push(std::pair<int, int>(lists[i]->val, i));
		}
	}
	return preHead.next;
}

// --------------------------------------------------------------------------------
/**
 * 5. Intersection of Two Linked Lists [160]
 *
 * Write a program to find the node at which the intersection of two singly linked lists begins.
 * @example
 *     The following two linked lists:
 *     A:          a1 → a2
 *                        ↘
 *                          c1 → c2 → c3
 *                        ↗            
 *     B:     b1 → b2 → b3
 *     begin to intersect at node c1.
 * @endexample
 * Note:
 * 1. If the two linked lists have no intersection at all, return null.
 * 2. The linked lists must retain their original structure after the function returns.
 * 3. You may assume there are no cycles anywhere in the entire linked structure.
 * 4. Your code should preferably run in O(n) time and use only O(1) memory.
 */

/**
 * We can use two iterations to do that. In the first iteration, we will reset the pointer
 * of one linkedlist to the head of another linkedlist after it reaches the tail node.
 * In the second iteration, we will move two pointers until they points to the same node.
 * Our operations in first iteration will help us counteract the difference. So if two
 * linkedlist intersects, the meeting point in second iteration must be the intersection
 * point. If the two linked lists have no intersection at all, then the meeting pointer
 * in second iteration must be the tail node of both lists, which is null.
 */
ListNode* getIntersectionNode(ListNode *headA, ListNode *headB)
{
	if (headA == NULL || headB == NULL)
		return NULL;

	ListNode *p = headA, *q = headB;
	while (p != q)
	{
		p = p != NULL ? p->next : headB;
		q = q != NULL ? q->next : headA;
	}
	return p;
}





///////////////////////////////////////////////////////////////////////////////////
// Category: Miscellany
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Palindrome Linked List [234]
 *
 * Given a singly linked list, determine if it is a palindrome.
 * Follow up:
 * Could you do it in O(n) time and O(1) space?
 */
class Solution
{
public:
	bool isPalindrome(ListNode *head)
	{
		s = head;
		return check(head);
	}

private:
	bool check(ListNode *t)
	{
		if (t == NULL)
			return true;
		bool palindrome = check(t->next) && s->val == t->val;
		s = s->next;
		return palindrome;
	}

	ListNode *s;
};

// --------------------------------------------------------------------------------
/**
 * 2. Split Linked List in Parts [725]
 *
 * Given a (singly) linked list with head node root, write a function to
 * split the linked list into k consecutive linked list "parts".
 * The length of each part should be as equal as possible: no two parts
 * should have a size differing by more than 1. This may lead to some
 * parts being null. The parts should be in order of occurrence in the
 * input list, and parts occurring earlier should always have a size
 * greater than or equal parts occurring later.
 * Return a List of ListNode's representing the linked list parts that are formed.
 * Examples 1->2->3->4, k = 5 // 5 equal parts [ [1], [2], [3], [4], null ]
 * @example
 *     Input:  root = [1, 2, 3], k = 5
 *     Output: [[1],[2],[3],[],[]]
 *     Explanation:
 *     The input and each element of the output are ListNodes, not arrays.
 *       For example, the input root has root.val = 1, root.next.val = 2, 
 *       root.next.next.val = 3, and root.next.next.next = null.
 *       The first element output[0] has output[0].val = 1, output[0].next = null.
 *       The last element output[4] is null, but it's string representation as a ListNode is [].
 * @endexample
 * @example
 *     Input: 
 *     root = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10], k = 3
 *     Output: [[1, 2, 3, 4], [5, 6, 7], [8, 9, 10]]
 *     Explanation:
 *     The input has been split into consecutive parts with size difference at most 1,
 *       and earlier parts are a larger size than the later parts.
 * @endexample
 * Note:
 * 1. The length of root will be in the range [0, 1000].
 * 2. Each value of a node in the input will be an integer in the range [0, 999].
 * 3. k will be an integer in the range [1, 50].
 */
std::vector<ListNode*> splitListToParts(ListNode *root, int k)
{
	int L = 0;
	for (ListNode *p = root; p != NULL; p = p->next)
		++L;
	int _n = L / k, _k = L % k;
	std::vector<ListNode*> res(k, NULL);
	ListNode *p = root, *q = NULL;
	for (int j = 0; p != NULL && j < k; ++j)
	{
		int n = j < _k ? _n + 1 : _n;
		res[j] = p;
		for (int i = 1; i < n; ++i)
			p = p->next;
		q = p->next;
		p->next = NULL;
		p = q;
	}
	return res;
}

// --------------------------------------------------------------------------------
/**
 * 3. Copy List with Random Pointer [138]
 *
 * A linked list is given such that each node contains an additional
 * random pointer which could point to any node in the list or null.
 * Return a deep copy of the list.
 */

/**
 * Definition for singly-linked list with a random pointer.
 * struct RandomListNode {
 *     int label;
 *     RandomListNode *next, *random;
 *     RandomListNode(int x) : label(x), next(NULL), random(NULL) {}
 * };
 */
RandomListNode* copyRandomList(RandomListNode *head)
{
	RandomListNode *p = head, *q = NULL;
	while (p != NULL)
	{
		q = new RandomListNode(p->label);
		q->next = p->next;
		p->next = q;
		p = q->next;
	}
	p = head;
	while (p != NULL)
	{
		q = p->next;
		q->random = p->random != NULL ? p->random->next : NULL;
		p = q->next;
	}
	p = head;
	RandomListNode *dupHead = head != NULL ? head->next : NULL;
	while (p != NULL)
	{
		q = p->next;
		p->next = q->next;
		p = p->next;
		q->next = p != NULL ? p->next : NULL;
	}
	return dupHead;
}
