/**
 * Definition for singly-linked list.
 * struct ListNode {
 *     int val;
 *     ListNode *next;
 *     ListNode(int x) : val(x), next(NULL) {}
 * };
 */
///////////////////////////////////////////////////////////////////////////////////
// Category: Removal
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Delete Node in a Linked List [237]
 *
 * Write a function to delete a node (except the tail) in a singly linked list,
 * given only access to that node.
 * Supposed the linked list is 1 -> 2 -> 3 -> 4 and you are given the third node
 * with value 3, the linked list should become 1 -> 2 -> 4 after calling your function.
 */
void deleteNode(ListNode *node);

// --------------------------------------------------------------------------------
/**
 * 2. Remove Nth Node From End of List [19]
 *
 * Given a linked list, remove the n-th node from the end of list and return its head.
 * @example
 *     Given linked list: 1->2->3->4->5, and n = 2.
 *     After removing the second node from the end, the linked list becomes 1->2->3->5.
 * @endexample
 * Note:
 * Given n will always be valid. Try to do this in one pass.
 */
ListNode* removeNthFromEnd(ListNode *head, int n);

// --------------------------------------------------------------------------------
/**
 * 3. Remove Linked List Elements [203]
 *
 * Remove all elements from a linked list of integers that have value val.
 * @example
 *     Given: 1 -> 2 -> 6 -> 3 -> 4 -> 5 -> 6, val = 6
 *     Return: 1 -> 2 -> 3 -> 4 -> 5
 * @endexample
 */
ListNode* removeElements(ListNode *head, int val);

// --------------------------------------------------------------------------------
/**
 * 4. Remove Duplicates from Sorted List [83]
 *
 * Given a sorted linked list, delete all duplicates such that each element appear only once.
 * @example
 *     Given 1->1->2, return 1->2.
 * @endexample
 * @example
 *     Given 1->1->2->3->3, return 1->2->3.
 * @endexample
 */
ListNode* deleteDuplicates(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 5. Remove Duplicates from Sorted List II [82]
 *
 * Given a sorted linked list, delete all nodes that have duplicate numbers,
 * leaving only distinct numbers from the original list.
 * @example
 *     Given 1->2->3->3->4->4->5, return 1->2->5.
 * @endexample
 * @example
 *     Given 1->1->1->2->3, return 2->3.
 * @endexample
 */
ListNode* deleteDuplicates(ListNode *head);





///////////////////////////////////////////////////////////////////////////////////
// Category: Reorder
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Reverse Linked List [206]
 *
 * Reverse a singly linked list. A linked list can be reversed either
 * iteratively or recursively. Could you implement both?
 */
ListNode* reverseList(ListNode *head);

ListNode* reverseList(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 2. Reverse Linked List II [92]
 *
 * Reverse a linked list from position m to n. Do it in-place and in one-pass.
 * @example
 *     Given 1->2->3->4->5->NULL, m = 2 and n = 4,
 *     return 1->4->3->2->5->NULL.
 * @endexample
 * Note:
 * Given m, n satisfy the following condition:
 * 1 <= m <= n <= length of list.
 */
ListNode* reverseBetween(ListNode *head, int m, int n);

// --------------------------------------------------------------------------------
/**
 * 3. Swap Nodes in Pairs [24]
 *
 * Given a linked list, swap every two adjacent nodes and return its head.
 * @example
 *     Given 1->2->3->4, you should return the list as 2->1->4->3.
 * @endexample
 * Your algorithm should use only constant space. You may not modify the
 * values in the list, only nodes itself can be changed.
 */
ListNode* swapPairs(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 4. Reverse Nodes in k-Group [25]
 *
 * Given a linked list, reverse the nodes of a linked list k
 * at a time and return its modified list. k is a positive integer
 * and is less than or equal to the length of the linked list.
 * If the number of nodes is not a multiple of k then left-out nodes
 * in the end should remain as it is.
 * You may not alter the values in the nodes, only nodes itself may be changed.
 * Only constant memory is allowed.
 * For example,
 * Given this linked list: 1 -> 2 -> 3 -> 4 -> 5
 * For k = 2, you should return: 2 -> 1 -> 4 -> 3 -> 5
 * For k = 3, you should return: 3 -> 2 -> 1 -> 4 -> 5
 */
ListNode* reverseKGroup(ListNode *head, int k);

// --------------------------------------------------------------------------------
/**
 * 5. Rotate List [61]
 *
 * Given a list, rotate the list to the right by k places, where k is non-negative.
 * @example
 *     Given 1->2->3->4->5->NULL and k = 2,
 *     return 4->5->1->2->3->NULL.
 * @endexample
 */
ListNode* rotateRight(ListNode *head, int k);

// --------------------------------------------------------------------------------
/**
 * 6. Partition List [86]
 *
 * Given a linked list and a value x, partition it such that all nodes
 * less than x come before nodes greater than or equal to x.
 * You should preserve the original relative order of the nodes in each of the two partitions.
 * @example
 *     Given 1->4->3->2->5->2 and x = 3,
 *     return 1->2->2->4->3->5.
 * @endexample
 */
ListNode* partition(ListNode *head, int x);

// --------------------------------------------------------------------------------
/**
 * 7. Odd Even Linked List [328]
 *
 * Given a singly linked list, group all odd nodes together followed by the even nodes.
 * Please note here we are talking about the node number and not the value in the nodes.
 * You should try to do it in place.
 * The program should run in O(1) space complexity and O(nodes) time complexity.
 * @example
 *     Given 1->2->3->4->5->NULL,
 *     return 1->3->5->2->4->NULL.
 * @endexample
 * Note:
 * The relative order inside both the even and odd groups should remain as it was in the input.
 * The first node is considered odd, the second node even and so on ...
 */
ListNode* oddEvenList(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 8. Insertion Sort List [147]
 *
 * Sort a linked list using insertion sort.
 */
ListNode* insertionSortList(ListNode *head);






///////////////////////////////////////////////////////////////////////////////////
// Category: fast slow pointers
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Sort List [148]
 *
 * Sort a linked list in O(n log n) time using constant space complexity.
 */
ListNode* sortList(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 2. Linked List Cycle [141]
 *
 * Given a linked list, determine if it has a cycle in it.
 * Follow up:
 * Can you solve it without using extra space?
 */
bool hasCycle(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 3. Linked List Cycle II [142]
 *
 * Given a linked list, return the node where the cycle begins.
 * If there is no cycle, return null.
 * Note:
 * Do not modify the linked list.
 * Follow up:
 * Can you solve it without using extra space?
 */
ListNode* detectCycle(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 4. Reorder List [143]
 *
 * Given a singly linked list L: L0 -> L1 -> ... -> Ln-1 -> Ln,
 * reorder it to: L0 -> Ln -> L1 -> Ln-1 -> L2 -> Ln-2 -> ...
 * You must do this in-place without altering the nodes' values.
 * @example
 *     Given {1,2,3,4}, reorder it to {1,4,2,3}.
 * @endexample
 */
void reorderList(ListNode *head);





///////////////////////////////////////////////////////////////////////////////////
// Category: Two Lists
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Add Two Numbers [2]
 *
 * You are given two non-empty linked lists representing two non-negative integers.
 * The digits are stored in reverse order and each of their nodes contain a single digit.
 * Add the two numbers and return it as a linked list.
 * You may assume the two numbers do not contain any leading zero,
 * except the number 0 itself.
 * @example
 *     Input: (2 -> 4 -> 3) + (5 -> 6 -> 4)
 *     Output: 7 -> 0 -> 8
 *     Explanation: 342 + 465 = 807.
 * @endexample
 */
ListNode *addTwoNumbers(ListNode *l1, ListNode *l2);

// --------------------------------------------------------------------------------
/**
 * 2. Add Two Numbers II [445]
 *
 * You are given two non-empty linked lists representing two non-negative integers.
 * The most significant digit comes first and each of their nodes contain a single
 * digit. Add the two numbers and return it as a linked list. You may assume the
 * two numbers do not contain any leading zero, except the number 0 itself.
 * @example
 *     Input: (7 -> 2 -> 4 -> 3) + (5 -> 6 -> 4)
 *     Output: 7 -> 8 -> 0 -> 7
 * @endexample
 * Follow up:
 * What if you cannot modify the input lists? In other words, reversing the lists
 * is not allowed.
 */
ListNode* addTwoNumbers(ListNode *l1, ListNode *l2);

// --------------------------------------------------------------------------------
/**
 * 3. Merge Two Sorted Lists [21]
 *
 * Merge two sorted linked lists and return it as a new list.
 * The new list should be made by splicing together the nodes of the first two lists.
 * @example
 *     Input: 1->2->4, 1->3->4
 *     Output: 1->1->2->3->4->4
 * @endexample
 */
ListNode* mergeTwoLists(ListNode *l1, ListNode *l2);

// --------------------------------------------------------------------------------
/**
 * 4. Merge k Sorted Lists [23]
 *
 * Merge k sorted linked lists and return it as one sorted list.
 * Analyze and describe its complexity.
 */
ListNode* mergeKLists(std::vector<ListNode*>& lists);

// --------------------------------------------------------------------------------
/**
 * 5. Intersection of Two Linked Lists [160]
 *
 * Write a program to find the node at which the intersection of two singly linked lists begins.
 * @example
 *     The following two linked lists:
 *     A:          a1 → a2
 *                        ↘
 *                          c1 → c2 → c3
 *                        ↗            
 *     B:     b1 → b2 → b3
 *     begin to intersect at node c1.
 * @endexample
 * Note:
 * 1. If the two linked lists have no intersection at all, return null.
 * 2. The linked lists must retain their original structure after the function returns.
 * 3. You may assume there are no cycles anywhere in the entire linked structure.
 * 4. Your code should preferably run in O(n) time and use only O(1) memory.
 */
ListNode* getIntersectionNode(ListNode *headA, ListNode *headB);





///////////////////////////////////////////////////////////////////////////////////
// Category: Miscellany
///////////////////////////////////////////////////////////////////////////////////
/**
 * 1. Palindrome Linked List [234]
 *
 * Given a singly linked list, determine if it is a palindrome.
 * Follow up:
 * Could you do it in O(n) time and O(1) space?
 */
bool isPalindrome(ListNode *head);

// --------------------------------------------------------------------------------
/**
 * 2. Split Linked List in Parts [725]
 *
 * Given a (singly) linked list with head node root, write a function to
 * split the linked list into k consecutive linked list "parts".
 * The length of each part should be as equal as possible: no two parts
 * should have a size differing by more than 1. This may lead to some
 * parts being null. The parts should be in order of occurrence in the
 * input list, and parts occurring earlier should always have a size
 * greater than or equal parts occurring later.
 * Return a List of ListNode's representing the linked list parts that are formed.
 * Examples 1->2->3->4, k = 5 // 5 equal parts [ [1], [2], [3], [4], null ]
 * @example
 *     Input:  root = [1, 2, 3], k = 5
 *     Output: [[1],[2],[3],[],[]]
 *     Explanation:
 *     The input and each element of the output are ListNodes, not arrays.
 *       For example, the input root has root.val = 1, root.next.val = 2, 
 *       root.next.next.val = 3, and root.next.next.next = null.
 *       The first element output[0] has output[0].val = 1, output[0].next = null.
 *       The last element output[4] is null, but it's string representation as a ListNode is [].
 * @endexample
 * @example
 *     Input: 
 *     root = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10], k = 3
 *     Output: [[1, 2, 3, 4], [5, 6, 7], [8, 9, 10]]
 *     Explanation:
 *     The input has been split into consecutive parts with size difference at most 1,
 *       and earlier parts are a larger size than the later parts.
 * @endexample
 * Note:
 * 1. The length of root will be in the range [0, 1000].
 * 2. Each value of a node in the input will be an integer in the range [0, 999].
 * 3. k will be an integer in the range [1, 50].
 */
std::vector<ListNode*> splitListToParts(ListNode *root, int k);

// --------------------------------------------------------------------------------
/**
 * 3. Copy List with Random Pointer [138]
 *
 * A linked list is given such that each node contains an additional
 * random pointer which could point to any node in the list or null.
 * Return a deep copy of the list.
 */

/**
 * Definition for singly-linked list with a random pointer.
 * struct RandomListNode {
 *     int label;
 *     RandomListNode *next, *random;
 *     RandomListNode(int x) : label(x), next(NULL), random(NULL) {}
 * };
 */
RandomListNode* copyRandomList(RandomListNode *head);
